@@ -3,9 +3,13 @@ fileiobase provides general purpose NMR file IO functions and classes
 used by multiple nmrglue.fileio modules.
 """
 
+from __future__ import division
+
 import os
 import string
+import sys
 import itertools
+from functools import reduce
 
 import numpy as np
 
@@ -17,16 +21,16 @@ def create_blank_udic(ndim):
     udic = dict()
     udic["ndim"] = ndim
 
-    for i in xrange(ndim):
+    for i in range(ndim):
         d = dict()
         d["sw"] = 999.99        # spectral width in Hz
         d["complex"] = True     # Quadrature, True when dimension is complex
         d["obs"] = 999.99       # Observation frequency in MHz
         d["car"] = 999.99       # Carrier frequency in Hz
-        d["size"] = 1           # Number of points in dimension based on the
-                                # shape of the data array. As such the direct
-                                # dimension (-1) size is R|I, all indirect
-                                # dimensions are R+I
+        # Number of points in dimension based on the shape of the data array.
+        # As such the direct dimension (-1) size is R|I, all indirect
+        # dimensions are R+I
+        d["size"] = 1
 
         d["label"] = ["X", "Y", "Z", "A"][i]    # name of dimension
 
@@ -137,9 +141,9 @@ class unit_conversion():
             pts = self.__us2pts(val)
         else:
             raise ValueError("invalid unit type")
-        #if self._cplx:
-        #    return pts+round(pts)
-        #else:
+        # if self._cplx:
+        #     return pts+round(pts)
+        # else:
         return pts
 
     def __pnt2unit(self, val, units):
@@ -147,8 +151,8 @@ class unit_conversion():
         Convert points to units
         """
         units = units.upper()
-        #if self._cplx:
-        #    val = val-round(val)
+        # if self._cplx:
+        #     val = val-round(val)
         if units == "PPM":
             k = self.__pts2ppm(val)
         elif units == "HZ":
@@ -177,10 +181,10 @@ class unit_conversion():
         """
         Convert string or value/unit pair
         """
-        if type(val) == str:
+        if isinstance(val, str):
             return self.__str2pnt(val)
         else:
-            if unit == None:
+            if unit is None:
                 raise ValueError("invalid unit type")
             return self.__unit2pnt(val, unit)
 
@@ -326,7 +330,31 @@ class unit_conversion():
     __call__ = i    # calling the object x is the same as x.i
 
 
-def open_towrite(filename, overwrite=False):
+def uc_from_udic(udic, dim=-1):
+    """
+    Create a unit conversion object from a Universal dictionary.
+
+    Parameters
+    ----------
+    udic : dic
+        Universal dictionary of spectral parameters.
+    dim : int. optional
+        Dimension number to create unit conversion object for.  Default is for
+        last dimension.
+
+    Returns
+    -------
+    uc : unit conversion object.
+        Unit conversion object for given dimension.
+    """
+    if dim == -1:
+        dim = udic['ndim'] - 1  # last dimension
+    adic = udic[dim]
+    return unit_conversion(adic['size'], adic['complex'], adic['sw'],
+                           adic['obs'], adic['car'])
+
+
+def open_towrite(filename, overwrite=False, mode='wb'):
     """
     Open filename for writing and return file object
 
@@ -334,15 +362,15 @@ def open_towrite(filename, overwrite=False):
     creates necessary directiories as needed.
     """
     # check if file exists and overwrite if False
-    if os.path.exists(filename) and (overwrite == False):
+    if os.path.exists(filename) and (overwrite is False):
         raise IOError("File exists, recall with overwrite=True")
 
     p, fn = os.path.split(filename)  # split into filename and path
     # create directories if needed
-    if p != '' and os.path.exists(p) == False:
+    if p != '' and os.path.exists(p) is False:
         os.makedirs(p)
 
-    return open(filename, 'wb')
+    return open(filename, mode)
 
 ################################################
 # numpy ndarray emulation and helper functions #
@@ -419,7 +447,7 @@ def index2trace_opp(shape, index):
     nphase = index2trace_flat([2] * n, phases[::-1])
     # deal with the remainer
     pindex = [v // 2 for v in index]
-    pshape = [i / 2 for i in shape]
+    pshape = [i // 2 for i in shape]
     nbase = index2trace_flat(pshape, pindex)
     return nbase * 2 ** n + nphase
 
@@ -432,7 +460,7 @@ def trace2index_opp(shape, ntrace):
     n = len(shape)
     q, r = divmod(ntrace, 2 ** n)
     to_add = list(trace2index_flat([2] * n, r))[::-1]
-    pshape = [i / 2 for i in shape]
+    pshape = [i // 2 for i in shape]
     base = list(trace2index_flat(pshape, q))
     total = [b * 2 + a for b, a in zip(base, to_add)]
     return tuple(total)
@@ -449,7 +477,7 @@ def index2trace_reg(shape, index):
     nphase = index2trace_flat([2] * n, phases)
     # deal with the remainer
     pindex = [v // 2 for v in index]
-    pshape = [i / 2 for i in shape]
+    pshape = [i // 2 for i in shape]
     nbase = index2trace_flat(pshape, pindex)
     return nbase * 2 ** n + nphase
 
@@ -462,7 +490,7 @@ def trace2index_reg(shape, ntrace):
     n = len(shape)
     q, r = divmod(ntrace, 2 ** n)
     to_add = list(trace2index_flat([2] * n, r))
-    pshape = [i / 2 for i in shape]
+    pshape = [i // 2 for i in shape]
     base = list(trace2index_flat(pshape, q))
     total = [b * 2 + a for b, a in zip(base, to_add)]
     return tuple(total)
@@ -545,7 +573,7 @@ class data_nd(object):
         x.__getitem__(y) <==> x[y]
         """
         # convert the key into a list
-        if type(key) != tuple:
+        if not isinstance(key, tuple):
             rlist = [key]
         else:
             rlist = list(key)
@@ -562,7 +590,7 @@ class data_nd(object):
 
         # replace none-slices with slice objects
         for i, v in enumerate(rlist):
-            if type(v) != slice:
+            if not isinstance(v, slice):
 
                 # check for out of range indexes
                 if v >= self.shape[i]:
@@ -603,7 +631,7 @@ class data_nd(object):
 
     def __iter__(self):
         """ x.__iter__() <==> iter(x) """
-        for index in xrange(0, self.shape[0]):
+        for index in range(0, self.shape[0]):
             yield self[index]
 
     def swapaxes(self, axis1, axis2):
